import { useDocument, useDocumentOnce } from 'react-firebase-hooks/firestore';
import { useDocumentSnapData } from './useSnapData.js';
import { useFirestoreErrorLogger, useRefChangeLimitExceeded } from './utils.js';
export const useTypedDocumentOnce = (typedDoc, { getOptions, ...dataOptions } = {}) => {
    const { safeRef } = useRefChangeLimitExceeded(typedDoc === null || typedDoc === void 0 ? void 0 : typedDoc.raw);
    const [_snap, loading, error] = useDocumentOnce(safeRef(), {
        getOptions,
    });
    useFirestoreErrorLogger(error);
    const { snap, data } = useDocumentSnapData(typedDoc, _snap, dataOptions);
    return { snap, data, loading, error };
};
export const useTypedDocument = (typedDoc, { snapshotListenOptions, ...dataOptions } = {}) => {
    const { safeRef } = useRefChangeLimitExceeded(typedDoc === null || typedDoc === void 0 ? void 0 : typedDoc.raw);
    const [_snap, loading, error] = useDocument(safeRef(), {
        snapshotListenOptions,
    });
    useFirestoreErrorLogger(error);
    const { snap, data } = useDocumentSnapData(typedDoc, _snap, dataOptions);
    return { snap, data, loading, error };
};
